
From: Kaigai Kohei <kaigai@ak.jp.nec.com>

This attached patch adds list_replace_rcu() to include/linux/list.h for
atomic updating operations according to RCU-model.

void list_replace_rcu(struct list_head *old, struct list_head *new)

The 'old' element is detached from the linked list, and the 'new' element
is inserted to the same point of the linked list concurrently.

This patch is necessary for the performance improvement of SELinux.
See, http://lkml.org/lkml/2004/8/16/54
       (Subject: RCU issue with SELinux)
     http://lkml.org/lkml/2004/8/30/63
       (Subject: [PATCH]SELinux performance improvement by RCU)

Signed-off-by: KaiGai, Kohei <kaigai@ak.jp.nec.com>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/include/linux/list.h |   15 +++++++++++++++
 1 files changed, 15 insertions(+)

diff -puN include/linux/list.h~list_replace_rcu-in-include-linux-listh include/linux/list.h
--- 25/include/linux/list.h~list_replace_rcu-in-include-linux-listh	2004-09-17 04:28:03.087825000 -0700
+++ 25-akpm/include/linux/list.h	2004-09-17 04:28:03.091824392 -0700
@@ -198,6 +198,21 @@ static inline void list_del_rcu(struct l
 	entry->prev = LIST_POISON2;
 }
 
+/*
+ * list_replace_rcu - replace old entry by new one
+ * @old : the element to be replaced
+ * @new : the new element to insert
+ *
+ * The old entry will be replaced with the new entry atomically.
+ */
+static inline void list_replace_rcu(struct list_head *old, struct list_head *new){
+	new->next = old->next;
+	new->prev = old->prev;
+	smp_wmb();
+	new->next->prev = new;
+	new->prev->next = new;
+}
+
 /**
  * list_del_init - deletes entry from list and reinitialize it.
  * @entry: the element to delete from the list.
_
